package com.example.jersey5156.rest;

import javax.inject.Singleton;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.glassfish.jersey.internal.inject.AbstractBinder;
import org.glassfish.jersey.server.ResourceConfig;
import org.glassfish.jersey.server.spi.Container;
import org.glassfish.jersey.server.spi.ContainerLifecycleListener;

@Singleton
public class RestApplication extends ResourceConfig {
    
    private static final Logger LOG = LogManager.getLogger(RestApplication.class);
    
    private static final RestApplicationListener LISTENER = new RestApplicationListener();
    
    public RestApplication() {
        configure();
    }

    private void configure() {
        setApplicationName("Reproducer for https://github.com/eclipse-ee4j/jersey/issues/5156");
        packages(RestApplication.class.getPackageName());
        registerInstances(LISTENER);
        registerInstances(new AbstractBinder() {
            @Override
            protected void configure() {
                bind(RestApplication.class).to(RestApplication.class);
            }
        });
    }
    
    public final void reload() {
        LOG.info("Application: container is about to be reloaded.");
        LISTENER.container.reload();
        LOG.info("Application: container was reloaded.");
    }
    
    private static class RestApplicationListener implements ContainerLifecycleListener {
        
        private Container container;

        @Override
        public void onStartup(Container container) {
            LOG.info("Listener: container was successfully started.");
            this.container = container;
        }

        @Override
        public void onReload(Container container) {
            LOG.info("Listener: container was successfully reloaded.");
            this.container = container;
        }

        @Override
        public void onShutdown(Container container) {
            LOG.info("Listener: container was successfully shut down.");
            this.container = container;
        }
    }

}
